@@ -1,15 +1,19 @@
 import os
+import re
 import socket
 import warnings
 from itertools import chain
 from pathlib import Path
-from typing import Dict, Sequence, Tuple, Callable, Any
+from typing import Dict, Sequence, Tuple, Callable, Any, Union
 import importlib
 
 from paramiko.config import SSHConfig
 from pydantic import BaseModel, Extra, validator, root_validator
 from yaml import safe_load
+
+from connectome.interface.blocks import CacheIndex
 from connectome.storage import Storage, SSHLocation, Disk
+from connectome.storage.config import HashConfig
 
 from .utils import PathLike
 
@@ -20,8 +24,55 @@ class ConfigError(Exception):
     pass
 
 
+class HostName:
+    key: str
+
+    def match(self, name) -> bool:
+        raise NotImplementedError
+
+    @classmethod
+    def __get_validators__(cls):
+        yield cls.validate
+
+    @classmethod
+    def validate(cls, v):
+        if isinstance(v, HostName):
+            return v
+        if isinstance(v, str):
+            return StrHostName(v)
+
+        assert isinstance(v, dict), f'Not a dict: {v}'
+        assert len(v) == 1
+        (k, v), = v.items()
+        for kls in HostName.__subclasses__():
+            if kls.key == k:
+                return kls(v)
+
+        raise ValueError(f'Invalid key "{k}" for hostname')
+
+
+class StrHostName(HostName):
+    key = 'str'
+
+    def __init__(self, value):
+        self.value = value
+
+    def match(self, name) -> bool:
+        return name == self.value
+
+
+class RegexHostName(HostName):
+    key = 'regex'
+
+    def __init__(self, pattern):
+        self.pattern = re.compile(pattern)
+
+    def match(self, name) -> bool:
+        return self.pattern.match(name) is not None
+
+
 class LocationConfig(BaseModel):
-    root: str
+    root: Path
     ssh: str = None
 
     @root_validator(pre=True)
@@ -37,16 +88,33 @@ class LocationConfig(BaseModel):
         extra = Extra.forbid
 
 
+LocationConfigs = Tuple[LocationConfig, ...]
+
+
 class StorageConfig(BaseModel):
     name: str
     default: Dict[str, Any] = None
-    hostname: Tuple[str, ...] = None
-    storage: Tuple[LocationConfig, ...]
-    cache: str = None
+    hostname: Tuple[HostName, ...] = ()
+    storage: LocationConfigs
+    cache: LocationConfigs = ()
 
-    @validator('storage', each_item=True, pre=True)
+    @validator('hostname', pre=True)
+    def from_single(cls, v):
+        if isinstance(v, (str, dict)):
+            v = v,
+        return v
+
+    @validator('storage', 'cache', pre=True)
+    def from_string(cls, v):
+        if isinstance(v, str):
+            v = v,
+        return v
+
+    @validator('storage', 'cache', each_item=True, pre=True)
     def add_defaults(cls, v, values):
         default = (values['default'] or {}).copy()
+        if isinstance(v, str):
+            v = {'root': v}
         assert isinstance(v, dict)
         default.update(v)
         return default
@@ -59,6 +127,13 @@ class ConfigMeta(BaseModel):
     choose: str = None
     fallback: str = None
     order: str = None
+    hash: Union[str, HashConfig] = None
+
+    @validator('hash', pre=True)
+    def normalize_hash(cls, v):
+        if isinstance(v, str):
+            v = {'name': v}
+        return v
 
     @root_validator(pre=True)
     def resolve_deprecation(cls, values):
@@ -82,11 +157,15 @@ class RepositoryConfig(BaseModel):
         extra = Extra.forbid
 
 
-def build_storage(root: Path) -> Tuple[Storage, str]:
-    with open(root / CONFIG, 'r') as file:
-        config = parse(root, safe_load(file))
+def load_config(config: Path) -> RepositoryConfig:
+    with open(config, 'r') as file:
+        return parse(config, safe_load(file))
 
+
+def build_storage(root: Path) -> Tuple[Storage, CacheIndex]:
+    config = load_config(root / CONFIG)
     meta = config.meta
+
     order_func: Callable[[Sequence[Disk]], Sequence[Disk]] = identity
     if meta.order is not None:
         path, attr = meta.order.rsplit('.', 1)
@@ -109,8 +188,9 @@ def build_storage(root: Path) -> Tuple[Storage, str]:
                     'hostname': location.ssh} or location.ssh in ssh_config.get_hostnames())
             ]
 
+    cache = config.local.cache
     loc = order_func([Disk(location.root) for location in config.local.storage])
-    return Storage(loc, remote), config.local.cache
+    return Storage(loc, remote), CacheIndex([c.root for c in cache], [])
 
 
 def parse(root, config) -> RepositoryConfig:
@@ -120,8 +200,10 @@ def parse(root, config) -> RepositoryConfig:
     meta = ConfigMeta.parse_obj(config.pop('meta', {}))
     entries = {}
     for name, entry in config.items():
+        if isinstance(entry, str):
+            entry = {'storage': entry}
         if not isinstance(entry, dict):
-            raise ConfigError('Each config entry must be a dict')
+            raise ConfigError('Each config entry must be either a dict or a string')
         if 'name' in entry:
             raise ConfigError('The key "name" is not available')
         entry = entry.copy()
@@ -165,8 +247,8 @@ def default_choose(meta: StorageConfig):
         return meta.name == os.environ[repo_key]
 
     node = socket.gethostname()
-    hosts = meta.hostname or [meta.name]
-    return any(h == node for h in hosts)
+    hosts = meta.hostname or [StrHostName(meta.name)]
+    return any(h.match(node) for h in hosts)
 
 
 def identity(x):
